"""
Sneak: A terminal snake game

Usage: sneak [options]

Options:
  -s SIZE, --size=SIZE  Game size (s | m | l)
  -h, --help            Show this help message and exit
  -v, --version         Show version and exit
"""

__title__ = "sneak-game"
__version__ = "0.2.0"
__license__ = "MIT"
